#ifndef CONSTANTS_H
#define CONSTANTS_H

#include <iostream>
#include <errno.h>
#include <stdio.h>
#include <limits>
#include <string>
#include <vector>
#include <deque>

// Transmittion and hardware specific settings
// When changed whole program has to be recompiled

#define RECEIVER 1
const double RECEIVER_COMMAND_TIMEOUT = 2;
const double RECEIVER_DATA_TIMEOUT = 0.5;
const std::string GPS_PACKET = "\\$GNGGA.*";

//#define TRANSMITTER
//const std::string GPS_PACKET = "\\$GPGGA.*";

// Debug options
//#define DEBUG 0b0
//#define DEBUG_GPS 0b1
//#define DEBUG_LOG 0b10
//#define DEBUG_MAG 0b11
//#define DEBUG_TRC 0b100
#define RELEASE

const int ERROR_SUCCESS = 1;
const int ERROR_FAILURE = 99;
const int ERROR_INIT_FAILED = 100;
const int ERROR_SERIAL_PORT_CLOSED = 101;
const int ERROR_DATA_NOT_RECOGNIZED = 102;
const int ERROR_DEVICE_NOT_RECOGNIZED = 103;
const int ERROR_CONNECTION_TIMEOUT = 104;
const int ERROR_DATA_NOT_READY = 105;

// ERROR codes for the transmittion
const int ERROR_STOP = 7;
const int ERROR_RESTART = 8;

const std::string DEV_FOLDER = "/dev/";
const std::string TTY_ACM = "ttyACM";
const std::string TTY_USB = "ttyUSB";
const std::string TTY_SERIAL0 = "serial0";

const std::string STOP = "stop";
const std::string LOG = "log";
const std::string SEND = "send";
const std::string RESTART = "pause";
const std::string ACK = "acknowledge";

const std::string PACKET_START = "##";
const std::string PACKET_END = "%%";
const char BODY_DELIMITER = '*';
const char DATA_DELIMITER = '&';

const int GPS_MIN_LENGTH = 12;		// extracting: 205811.00,5537.29849,N,01234.37894,E - encoded into three floats
const int MAG_MIN_LENGTH = 8;       // extraction: 050400.3635 1 23.5  - encoded into two floats

const int RETRANSMITTION_LIMIT = 1;

enum BAUD_RATE
{
	COM_BAUD_1200 = 1200,
	COM_BAUD_2400 = 2400,
	COM_BAUD_4800 = 4800,
	COM_BAUD_9600 = 9600,
	COM_BAUD_19200 = 19200,
	COM_BAUD_38400 = 38400,
	COM_BAUD_57600 = 57600,
	COM_BAUD_115200 = 115200,
};

enum TRANSMITTER_STATE
{
	TRANSMITTER_READY = 1,
	TRANSMITTER_SEND,
	TRANSMITTER_LOG,
	TRANSMITTER_RESET,
	TRANSMITTER_TERMINATE,
};

enum RECEIVER_STATE
{
	RECEIVER_READY = 1,
	RECEIVER_TRANSMIT,
	RECEIVER_RECEIVE,
	RECEIVER_RESET,
	RECEIVER_TERMINATE,
	RECEIVER_WAIT
};

// Recognition of packet
enum TRANSMITTION_STATE
{
	FIND_START,
	FIND_END
};

enum TRANSMITTION_PRIORITY
{
	DATA,
	COMMAND
};

#endif
