#ifndef DEVICE_H
#define DEVICE_H

#include <regex>
#include <cstring>
#include <unistd.h>
#include <wiringPi.h>
#include <wiringSerial.h> 

#include "constants.h"


/*
 * Class device is the parent class for all possible
 * devices, which could be attached during the experiments
 */
 
class Device
{
public:
	Device(){}
	~Device(){}
	/* 
	 * device identifier is the device identifier within the
	 * the /dev folder of the RPI. It is used to open communication
	 * and to obtain the file descriptor
	 */
	virtual void terminate_connection() = 0;
	virtual int init_device(std::string device) = 0;
	virtual int process_received_data() = 0;

	virtual int receive_data(); 
	virtual bool data_available();
	virtual int open_port(std::string device, BAUD_RATE);
	
	std::string get_error_message() 
			{return __error_message;}
			
	std::string get_to_log()
			{return __to_log;}
			
	std::string get_to_transfer()
			{return __to_transfer;}		
	
protected:
	int __file_descriptor;
	std::string __buffer;			// buffer for receiving data from given file descriptor
	std::string __to_log;           // raw string got from device, which is logged onboard to file
	std::string __to_transfer;      // extracted and encoded data to transfer
	std::string __error_message;
};

#endif
