#ifndef RECEIVER_H
#define RECEIVER_H

#include <mutex>
#include <thread>
#include <chrono>
#include <fstream>
#include <iostream>
#include <sys/stat.h>

#include "constants.h"
#include "transciever.h"

// Functions for communication with Python processing
void data_thread();
void command_thread();

std::string get_data();
std::string get_command();

void set_data(std::string t_data);
void set_command(std::string t_command);

/*
 * Receiver class is the top level class using 
 * transceiver device for communication with Transmitter
 * and getters and setters for communication with Python
 */
class Receiver
{
public:
	// Checks whether transceiver is working as intended
	int init_devices();
	// Main function which runs until request for termination is send from user
	int run_loop();
	
	std::string get_error_message()
		{return __error_message;}
		
private:
	// React on command from user and switch the state in which the Receiver class is 
	int react_on_command(std::string cmd);
	int process_received_data(std::string & data);
	void process_data_to_send(std::string & command);
	// Used for sending the command to Transmitter and waiting for the response
	void handle_command(std::string command);
private:
	std::string __error_message;
	Transciever * __transciever;
	RECEIVER_STATE __state;

};

#endif
