#ifndef CONTROLLER_H
#define CONTROLLER_H

#include "magnetometer.h"
#include "transciever.h"
#include "constants.h"
#include "logger.h"
#include "gps.h"


#include <numeric>
#include <dirent.h>
#include <chrono>
#include <ctime>

/*
 * Transmitter class is the top level class
 * gathering data from all the peripherals and
 * send them to the Receiver when requested
 */ 

class Transmitter
{

public:
	// Main function which runs until the request for termination is received
	int run_loop();
	// Check which peripherals are connected same as initialize them and the log file
	int init_devices();

	std::string get_error_message()
		{return __error_message;}

private:
	// Closes the connection
	void terminate_connection();
	// Generate the name of the log file in format complete timestamp
	std::string construct_name();
	
	int process_received_data();		
	void process_data_to_send(std::vector<std::string> command, TRANSMITTION_PRIORITY priority = DATA);	
	
private:
	/* 
	 * All peripheral devices recognized such as GPS or Magnetometer.
	 * As they all inherit from same class, 
	 * they can be stored in one vector together. 
	 */ 
	std::deque<Device*> __input_devices;
	std::string __error_message;
	Transciever * __transciever;
	Logger * __logger;

	// Vector of messages to be transmitted from each of the input devices
	std::vector<std::string> __to_transfer;
	// Vectr of messages to be logged from each of the input devices
	std::vector<std::string> __to_log;
	// Last received message from the Receiver
	std::string __last_received;
	TRANSMITTER_STATE __state;
};

#endif
