#include "../include/magnetometer.h"

/*
 * Initialization of the magnetometer devices connected
 * We try to open the port. If succesful we read the first packet 
 * and then the initialization is complete 
 */
int Magnetometer::init_device(std::string path)
{
	int port_status = open_port(path, COM_BAUD_115200);
	if(port_status != ERROR_SUCCESS)
		return ERROR_SERIAL_PORT_CLOSED;

	while(!data_available());
	if(Device::receive_data() > 0)
		return ERROR_SUCCESS;

	return ERROR_DEVICE_NOT_RECOGNIZED;
}

// Looking for end of line character. If it is found data is processed
int Magnetometer::process_received_data()
{
	if(__buffer.back() == '\n')
	{
#ifdef DEBUG_MAG
		std::cout << "Magnetometer received " << __buffer << std::endl;
		std::cout << "Size of data: " << __buffer.size() << std::endl;
#endif
		__to_log = __buffer;
		__to_transfer = __buffer;
		__buffer.clear();

		if(extract_data_to_transfer(__to_transfer))
			return ERROR_SUCCESS;
	}
	return ERROR_DATA_NOT_RECOGNIZED;

}

/*
 * Separating string into vector by the space character.
 * Then the conversion of the string segments to bytes is called
 */
bool Magnetometer::extract_data_to_transfer(std::string & message)
{
	std::stringstream separator(message);
	std::vector<std::string> segment_list;
	std::string segment;
	std::vector<uint8_t> output;
	
	while(std::getline(separator,segment,' '))
		segment_list.push_back(segment);

	segment_list.erase(segment_list.begin());
	segment_list.resize(9);
	convert_number_to_byte(segment_list[0],output,1);
	float sign = (segment_list[1].find("1") != std::string::npos) ? 1 : -1; // To save four byte os transmitted data
	convert_number_to_byte(segment_list.back(),output, sign);

#ifdef DEBUG_MAG
	std::cout << "Extracter mag data: " << segment << std::endl;
#endif
	message.assign(output.begin(),output.end());
	return (message.size() >= MAG_MIN_LENGTH) ? true : false;
}

/*
 * Conversion of the string representation of the numbers to the byte one.
 */
void Magnetometer::convert_number_to_byte(std::string message, std::vector<uint8_t> & output, float sign)
{
	float number = sign * std::stof(message);
	uint8_t const *pointer = reinterpret_cast<uint8_t const *>(&number);
	for(size_t i =0; i < sizeof(float); i++)
		output.push_back(pointer[i]);	
}

void Magnetometer::terminate_connection()
{
	serialClose(__file_descriptor);
}
